var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var fetch_exports = {};
__export(fetch_exports, {
  default: () => fetch_default,
  fetchRequestStream: () => fetchRequestStream,
  fetchResponseStream: () => fetchResponseStream,
  fetchSetDefaults: () => fetchSetDefaults
});
module.exports = __toCommonJS(fetch_exports);
var import_core = require("@datastream/core");
const defaults = {
  rateLimit: 0.01,
  rateLimitTimestamp: 0,
  dataPath: void 0,
  nextPath: void 0,
  qs: void 0,
  method: "GET",
  headers: {
    Accept: "application/json",
    "Accept-Encoding": "br, gzip, deflate"
  }
};
const fetchSetDefaults = (options) => {
  const headers = { ...defaults.headers, ...options.headers };
  Object.assign(defaults, options, { headers });
};
const fetchRequestStream = async (options, streamOptions = {}) => {
  const body = (0, import_core.createReadableStream)();
  const value = await fetch(options.url, {
    ...options,
    body,
    duplex: "half",
    signal: streamOptions.signal
  });
  const write = (chunk) => {
    body.push(chunk);
  };
  const stream = (0, import_core.createWritableStream)(write, streamOptions);
  stream.result = () => ({ key: "output", value });
};
const fetchResponseStream = (fetchOptions, streamOptions = {}) => {
  if (!Array.isArray(fetchOptions))
    fetchOptions = [fetchOptions];
  return (0, import_core.createReadableStream)(
    fetchGenerator(fetchOptions, streamOptions),
    streamOptions
  );
};
async function* fetchGenerator(fetchOptionsArray, streamOptions) {
  const requests = [];
  for (let options of fetchOptionsArray) {
    options.headers = { ...defaults.headers, ...options.headers };
    options = { ...defaults, ...options };
    if (options.qs) {
      options.url += ("?" + new URLSearchParams(options.qs)).replaceAll(
        "+",
        "%20"
      );
    }
    if (typeof options.dataPath !== "undefined" && /application\/([a-z.]+\+|)json/.test(options.headers.Accept)) {
      requests.push(fetchJson(options, streamOptions));
    } else {
      requests.push(fetchBody(options, streamOptions));
    }
  }
  for (const request of requests) {
    const response = await request;
    for await (const chunk of response) {
      yield chunk;
    }
  }
}
const fetchBody = async (options, streamOptions) => {
  const response = await fetchRateLimit(options, streamOptions);
  return response.body;
};
const nextLinkRegExp = /<(.*?)>; rel="next"/;
async function* fetchJson(options, streamOptions) {
  const { dataPath, nextPath } = options;
  while (options.url) {
    const response = await fetchRateLimit(options, streamOptions);
    const body = await response.json();
    options.url = nextPath && pickPath(body, nextPath);
    options.url ??= parseLink(response.headers);
    const data = pickPath(body, dataPath);
    if (Array.isArray(data)) {
      for (const item of data) {
        yield item;
      }
    } else {
      yield data;
    }
  }
}
const parseLink = (headers) => {
  const link = headers.get("Link");
  if (link) {
    return link.match(nextLinkRegExp)?.[1];
  }
};
const fetchRateLimit = async (options, { signal }) => {
  options.rateLimitTimestamp ??= 0;
  const now = Date.now();
  if (now < options.rateLimitTimestamp) {
    await (0, import_core.timeout)(options.rateLimitTimestamp - now, { signal });
  }
  options.rateLimitTimestamp = now + 1e3 * options.rateLimit;
  const response = await fetch(options.url, { ...options, signal });
  if (!response.ok) {
    if (response.statusCode === 429) {
      return fetchRateLimit(options, { signal });
    }
    throw new Error("fetch", {
      cause: { request: options, response }
    });
  }
  return response;
};
const pickPath = (obj, path) => {
  if (path === "")
    return obj;
  if (!Array.isArray(path))
    path = path.split(".");
  return path.slice(0).reduce((a, b) => {
    return a[b];
  }, obj);
};
var fetch_default = {
  setDefaults: fetchSetDefaults,
  responseStream: fetchResponseStream
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fetchRequestStream,
  fetchResponseStream,
  fetchSetDefaults
});
